@@ -1,15 +1,15 @@
+use crate::Opt;
 use crate::column::Column;
 use crate::columns::*;
 use crate::config::*;
+use crate::opt::{ArgColorMode, ArgPagerMode};
 use crate::process::collect_proc;
 use crate::style::{apply_color, apply_style, color_to_column_style};
 use crate::term_info::TermInfo;
 use crate::util::{
-    classify, find_column_kind, find_exact, find_partial, truncate, ArgColorMode, ArgPagerMode,
-    KeywordClass,
+    KeywordClass, ansi_trim_end, classify, find_column_kind, find_exact, find_partial, truncate,
 };
-use crate::Opt;
-use anyhow::{bail, Error};
+use anyhow::{Error, bail};
 #[cfg(not(target_os = "windows"))]
 use pager::Pager;
 use std::collections::HashMap;
@@ -114,13 +114,17 @@ impl View {
             };
 
             for kind in kinds {
-                let visible = if let Some(ref only) = opt.only {
-                    let kind_name = KIND_LIST[&kind].0.to_lowercase();
-                    if !kind_name.contains(&only.to_lowercase()) {
-                        false
-                    } else {
-                        only_kind_found = true;
+                let visible = if let Some(only) = &opt.only {
+                    if kind == ConfigColumnKind::Tree {
                         true
+                    } else {
+                        let kind_name = KIND_LIST[&kind].0.to_lowercase();
+                        if !kind_name.contains(&only.to_lowercase()) {
+                            false
+                        } else {
+                            only_kind_found = true;
+                            true
+                        }
                     }
                 } else {
                     true
@@ -133,6 +137,7 @@ impl View {
                     &config.display.separator,
                     config.display.abbr_sid,
                     &config.display.tree_symbols,
+                    opt.procfs.clone(),
                 );
                 if column.available() {
                     columns.push(ColumnInfo {
@@ -151,13 +156,16 @@ impl View {
         }
 
         if slot_idx < opt.insert.len() {
-            bail!("There is not enough slot for inserting columns {:?}.\nPlease add \"Slot\" or \"MultiSlot\" to your config.\nhttps://github.com/dalance/procs#insert-column", opt.insert);
+            bail!(
+                "There is not enough slot for inserting columns {:?}.\nPlease add \"Slot\" or \"MultiSlot\" to your config.\nhttps://github.com/dalance/procs#insert-column",
+                opt.insert
+            );
         }
 
-        if let Some(ref only_kind) = opt.only {
-            if !only_kind_found {
-                bail!("kind \"{}\" is not found in columns", only_kind);
-            }
+        if let Some(only_kind) = &opt.only
+            && !only_kind_found
+        {
+            bail!("kind \"{}\" is not found in columns", only_kind);
         }
 
         let show_thread = if opt.thread {
@@ -172,6 +180,7 @@ impl View {
             Duration::from_millis(opt.interval),
             show_thread,
             config.display.show_kthreads,
+            &opt.procfs,
         );
         for c in columns.iter_mut() {
             for p in &proc {
@@ -210,7 +219,7 @@ impl View {
         })
     }
 
-    pub fn filter(&mut self, opt: &Opt, config: &Config) {
+    pub fn filter(&mut self, opt: &Opt, config: &Config, header_lines: usize) {
         let mut cols_nonnumeric = Vec::new();
         let mut cols_numeric = Vec::new();
         for c in &self.columns {
@@ -321,7 +330,8 @@ impl View {
                 visible_pids.push(*pid);
             }
 
-            if opt.watch_mode && visible_pids.len() >= self.term_info.height - 5 {
+            let reserved_rows = 4 + header_lines;
+            if opt.watch_mode && visible_pids.len() >= self.term_info.height - reserved_rows {
                 break;
             }
         }
@@ -331,11 +341,11 @@ impl View {
     }
 
     fn get_parent_pids(&self, pid: i32, parent_pids: &mut Vec<i32>) {
-        if let Some(x) = self.parent_pids.get(&pid) {
-            if !parent_pids.contains(x) {
-                parent_pids.push(*x);
-                self.get_parent_pids(*x, parent_pids);
-            }
+        if let Some(x) = self.parent_pids.get(&pid)
+            && !parent_pids.contains(x)
+        {
+            parent_pids.push(*x);
+            self.get_parent_pids(*x, parent_pids);
         }
     }
 
@@ -372,6 +382,12 @@ impl View {
         config: &Config,
         theme: &ConfigTheme,
     ) -> Result<(), Error> {
+        if opt.json {
+            self.term_info.use_pager = false;
+            self.display_json()?;
+            return Ok(());
+        }
+
         let use_terminal = console::user_attended();
 
         // +3 means header/unit line and next prompt
@@ -386,7 +402,7 @@ impl View {
                 + self.columns.len()
                 - 1
         } else {
-            std::usize::MIN
+            usize::MIN
         };
 
         let use_builtin_pager = if cfg!(target_os = "windows") {
@@ -424,7 +440,7 @@ impl View {
         truncate |= !use_terminal && config.display.cut_to_pipe;
 
         if !truncate {
-            self.term_info.width = std::usize::MAX;
+            self.term_info.width = usize::MAX;
         }
 
         match (opt.color.as_ref(), &config.display.color_mode) {
@@ -497,7 +513,7 @@ impl View {
                 );
             }
         }
-        row = row.trim_end().to_string();
+        row = ansi_trim_end(&row);
         row = truncate(&row, self.term_info.width).to_string();
         self.term_info.write_line(&row)?;
         Ok(())
@@ -519,7 +535,7 @@ impl View {
                 );
             }
         }
-        row = row.trim_end().to_string();
+        row = ansi_trim_end(&row);
         row = truncate(&row, self.term_info.width).to_string();
         self.term_info.write_line(&row)?;
         Ok(())
@@ -548,12 +564,39 @@ impl View {
                 );
             }
         }
-        row = row.trim_end().to_string();
+        row = ansi_trim_end(&row);
         row = truncate(&row, self.term_info.width).to_string();
         self.term_info.write_line(&row)?;
         Ok(())
     }
 
+    fn display_json(&self) -> Result<(), Error> {
+        self.term_info.write_line("[")?;
+
+        let len_pid = self.visible_pids.len();
+        for (i, pid) in self.visible_pids.iter().enumerate() {
+            let mut line = "{".to_string();
+            let len_column = self.columns.len();
+            for (j, c) in self.columns.iter().enumerate() {
+                if c.visible && c.kind != ConfigColumnKind::Separator {
+                    let text = c.column.display_json(*pid);
+                    line.push_str(&text);
+                    if j != len_column - 1 {
+                        line.push_str(", ");
+                    }
+                }
+            }
+            line.push('}');
+            if i != len_pid - 1 {
+                line.push(',');
+            }
+            self.term_info.write_line(&line)?;
+        }
+
+        self.term_info.write_line("]")?;
+        Ok(())
+    }
+
     fn get_sort_info(opt: &Opt, config: &Config, cols: &[ColumnInfo]) -> SortInfo {
         let (mut sort_idx, sort_order) = match (&opt.sorta, &opt.sortd) {
             (Some(sort), _) | (_, Some(sort)) => {
@@ -668,7 +711,6 @@ impl View {
     #[cfg(target_os = "windows")]
     fn pager(_config: &Config) {}
 
-    #[cfg_attr(tarpaulin, skip)]
     pub fn inc_sort_column(&mut self) -> usize {
         let current = self.sort_info.idx;
         let max_idx = self.columns.len();
@@ -682,7 +724,6 @@ impl View {
         current
     }
 
-    #[cfg_attr(tarpaulin, skip)]
     pub fn dec_sort_column(&mut self) -> usize {
         let current = self.sort_info.idx;
         let max_idx = self.columns.len();
